# coding=utf8
# import allure
# import pytest
# import json
# from time import sleep
# from utils.extractor import extract
# # from testAPI.interface.image_stock import ImageStock
# # from testAPI.suite.test_suits.conftest import param_new_image
# from utils.sql import Sql
# from testAPI.suite.test_suits.conftest import *
# from utils.config import YamlParam, BASE_PATH
# from testAPI.suite.test_suits.test_11_attribute import env_add_attribute
# from testAPI.suite.test_suits.test_13_image import map_image_param
# import os


# def map_attribute_id(group_params):
#     # 把 group params 中的所有 attribute name 转化为 id
#     for group_param in group_params:
#         if isinstance(group_param["attributes"], list):
#             for attribute in group_param["attributes"]:
#                 if isinstance(attribute, dict) and attribute.get("id"):
#                     attribute["id"] = env_add_attribute(attribute["id"])
#     return group_params
#
#
# attribute_group_yml = 'data/input_loads/image_stock/image.yml'
# yp = YamlParam(attribute_group_yml)
# param_get_attribute_groups_valid = [tuple(i) for i in yp.valid_params("param_get_attribute_groups")]
# param_get_attribute_groups_invalid = [tuple(i) for i in yp.invalid_params("param_get_attribute_groups")]
# x = yp.invalid_params("param_add_attribute_group")
# param_add_attribute_group_valid = map_attribute_id(x)
# param_add_attribute_group_invalid = yp.invalid_params("param_add_attribute_group")
# param_get_attribute_group_valid = yp.valid_params("param_get_attribute_group")
# param_get_attribute_group_invalid = yp.invalid_params("param_get_attribute_group")
# param_edit_attribute_group_valid = yp.valid_params("param_edit_attribute_group")
# param_edit_attribute_group_invalid = yp.invalid_params("param_edit_attribute_group")
# param_delete_attribute_group_valid = yp.valid_params("param_delete_attribute_group")
# param_delete_attribute_group_invalid = yp.invalid_params("param_delete_attribute_group")
#
# param_add_attribute_group_invalid_special = yp.dict_value_to_list(yp.get('param_add_attribute_group_invalid_special'))
#
# print(param_add_attribute_group_valid)

# param_search_image_valid = yp.invalid_params("param_search_image")
# print(param_search_image_valid)

#
#
#
#
# param_add_image_valid = map_image_param(yp.valid_params("param_add_image"))
# param_add_image_invalid = map_image_param(yp.invalid_params("param_add_image"))
# param_get_image_valid = yp.valid_params("param_get_image")
# param_get_image_invalid = yp.invalid_params("param_get_image")
# param_edit_image_valid = map_image_param(yp.valid_params("param_edit_image"))
# param_edit_image_invalid = map_image_param(yp.invalid_params("param_edit_image"))
# param_delete_image_valid = yp.valid_params("param_delete_image")
# param_delete_image_invalid = yp.invalid_params("param_delete_image")
#
# param_search_image_valid = map_image_param(yp.valid_params("param_search_image"))
# param_search_image_invalid = map_image_param(yp.invalid_params("param_search_image"))
#
# print(param_add_image_valid)
# print(param_add_image_invalid)
# print(param_get_image_valid)
# print(param_get_image_invalid)
# print(param_edit_image_valid)
# print(param_edit_image_invalid)
# print(param_delete_image_valid)
# print(param_delete_image_invalid)

# param_search_image_special_nogroup_invalid = yp.invalid_params("param_search_image_special_nogroup")

# param_mass_upload_image_special_zip = yp.invalid_params("zip", template_file="data/templates/image_stock_mass_upload_image.json")
# mass_upload_zip_path = BASE_PATH + "data/mass_images"
# param_zip_path = [os.path.join(BASE_PATH, "data/mass_images", i) for i in yp.valid_params("zip")]
# mass_template = Config("data/templates/image_stock_mass_upload_image.json").json_get()
# print(param_zip_path[0].split('/')[-1][:30])
# print(mass_template)

#
# def db_attribute_group_info(attribute_id):
#     sql = Sql('sql_dl')
#     query_content = f'SELECT name, data_type, extra_data FROM image_stock_attribute_tab ' \
#                     f'WHERE id = {attribute_id};'
#     name, data_type, extra_data = sql.query(query_content)[0]
#     sql.close()
#     possible_values = json.loads(extra_data).get('possible_values')
#     attribute_info = {'name': name, 'data_type': data_type, 'possible_values': possible_values}
#     return attribute_info
#
#
# def pre_attribute_group_id(api, original_name='test_for_api'):
#     # prepare data: add attribute_id to edit
#     original_json = {"name": original_name, "data_type": 1}
#     sql = Sql('sql_dl')
#     query_content = f'SELECT id FROM image_stock_attribute_tab WHERE name = \'{original_name}\';'
#     if not sql.query(query_content):
#         api.new_attribute_group(json=original_json)
#     else:
#         api.attribute_id = sql.query(query_content)[0][0]
#     sql.close()
#
#
# @allure.severity('blocker')
# @allure.feature('ImageStock')
# @allure.story('Attributes')
# class TestAttributeGroup(object):
#
#     @allure.title('Get Attribute Groups Valid')
#     @pytest.mark.parametrize('offset, limit', param_get_attribute_groups_valid)
#     def test_get_attribute_groups_valid(self, offset, limit):
#         istk = ImageStock()
#         response = istk.list_groups(offset, limit)
#
#         res_attributes = extract("result.attribute_groups", response)
#         res_total = extract("result.total", response)
#         res_req = []
#         if res_attributes:
#             for req in res_attributes:
#                 id = req.get("id")
#                 name = req.get("name")
#                 res_req.append((id, name))
#
#         sql = Sql('sql_dl')
#         query_content = f'SELECT id, name FROM image_stock_attribute_group_tab ' \
#                         f'ORDER BY id DESC LIMIT {limit} OFFSET {offset};'
#         res_db = sql.query(query_content)
#         total = len(sql.query('SELECT id, name FROM image_stock_attribute_group_tab '))
#         sql.close()
#         assert set(res_req) == set(res_db)
#         assert res_total == total
#
#     @allure.title('Get Attributes Invalid')
#     @pytest.mark.parametrize('offset, limit', param_get_attribute_groups_invalid)
#     def test_get_attribute_groups_invalid(self, offset, limit):
#         istk = ImageStock()
#         response = istk.list_groups(offset, limit, status='FAIL')
#
#         req_status = extract("success", response)
#         assert req_status is False


#
# import json
#
# def env_get_image_info_from_db(image_id):
#     # 根据image id，在数据库中查找图片json信息
#     TABLENAME = "image_stock_image_tab_00000" + str(image_id % 1000)
#     ID = image_id // 1000
#     res_image_info = {}
#     sql = Sql('sql_dl')
#     query_image_id = f'SELECT name, hash, width, height, extension, attribute_group_id, extra_data ' \
#                      f'FROM {TABLENAME} ' \
#                      f'WHERE id = {ID}'
#     name, image_hash, width, height, extension, attribute_group_id, extra_data = sql.query(query_image_id)[0]
#     res_image_info['name'] = name
#     res_image_info['hash'] = image_hash
#     res_image_info['attribute_group_id'] = attribute_group_id
#     res_image_info['extension'] = extension
#     res_image_info['width'] = width
#     res_image_info['height'] = height
#     res_image_info['attributes'] = []
#
#     if extra_data:
#         attributes = json.loads(extra_data)["attributes"]
#         logger.info(attributes)
#         for attribute in attributes:
#             attr = {}
#             attr['id'] = attribute['id']
#             attr['value'] = attribute['value']
#             res_image_info['attributes'].append(attr)
#
#     sql.close()
#     return res_image_info
#
#
# print(env_get_image_info_from_db(1734))
#
# from src.api.utils.config import Config
#
# con = Config()
# print(con.get())

# data = '{"live_account_info": {"is_host": False, "is_banned": True}, "editable_username": False, "is_semi_inactive": True, "already_verified_phone": True, "gender": 1, "adult_consent": 1603289583, "tos_accepted_time": 1600757444, "access": {"wallet_setting": 1, "wallet_provider": 1}, "feed_account_info": {"can_post_feed": True}, "nickname": "Stonegogo"}'
#
# data = '{"live_account_info": {"is_host": "False", "is_banned": "True"}, "editable_username": "False", "is_semi_inactive": "True", "already_verified_phone": "True", "gender": 1, "adult_consent": 1603289583, "tos_accepted_time": 1600757444, "access": {"wallet_setting": 1, "wallet_provider": 1}, "feed_account_info": {"can_post_feed": "True"}, "nickname": "Stonegogo"}'
#
#
# data_1 = "{'live_account_info': " \
#          "{'is_host': False, 'is_banned': True}, " \
#          "'editable_username': False, 'is_semi_inactive': True, " \
#          "'already_verified_phone': True, 'gender': 1, 'adult_consent': 1603289583, 'tos_accepted_time': 1600757444, " \
#          "'access': {'wallet_setting': 1, 'wallet_provider': 1}, 'feed_account_info': {'can_post_feed': True}, " \
#          "'nickname': u'Stonegogo'}"

#
# info = json.loads(data)data_1
# print(type(info))

# data = eval(data_1)
# print(type(data))
# print(data)
# info = json.loads(eval(data_1))
# print(type(info))
#
#
# [
#             {
#                 "userid": 10661,
#                 "username": "bbbcccc",
#                 "shopid": 10661,
#                 "shop_name": "b",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905556,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 10661,
#                 "username": "bbbcccc",
#                 "shopid": 10661,
#                 "shop_name": "b",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905556,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 10661,
#                 "username": "bbbcccc",
#                 "shopid": 10661,
#                 "shop_name": "b",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905556,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 10661,
#                 "username": "bbbcccc",
#                 "shopid": 10661,
#                 "shop_name": "b",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905556,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 10661,
#                 "username": "bbbcccc",
#                 "shopid": 10661,
#                 "shop_name": "b",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905556,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 10661,
#                 "username": "bbbcccc",
#                 "shopid": 10661,
#                 "shop_name": "b",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905556,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 10661,
#                 "username": "bbbcccc",
#                 "shopid": 10661,
#                 "shop_name": "b",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905556,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 0,
#                 "username": "",
#                 "shopid": 10101,
#                 "shop_name": "blueshkblue",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905532,
#                 "brand_label": 1
#             },
#             {
#                 "userid": 0,
#                 "username": "",
#                 "shopid": 11330,
#                 "shop_name": "Testwj102",
#                 "logo": "7c1358eb1b0a57d9203fe631884d8d41",
#                 "logo_pc": "33ea569be630c8e5d86b1b4723f9f18a",
#                 "shop_collection_id": 0,
#                 "ctime": 1606905582,
#                 "brand_label": 1
#             }
#         ]


# def sql_condition_generate(sql_str, condition):
#     """根据条件，产生数据库查询条件语句"""
#     if not condition:
#         return
#
#     condition_list = []
#     condition_param = []
#
#     if condition["name"]:
#         _ = str(condition["name"]).strip()
#         if condition["IsFuzzy"]:
#             condition_list.append("INSTR(name, {})")
#             condition_param.append(repr(_))
#         else:
#             condition_list.append("name = {}")
#             condition_param.append(repr(_))
#
#     if condition["attribute_group_name"]:
#         _ = str(condition["attribute_group_name"]).strip()
#         if condition["IsFuzzy"]:
#             condition_list.append("INSTR(attribute_group_name, {})")
#             condition_param.append(repr(_))
#         else:
#             condition_list.append("attribute_group_name = {}")
#             condition_param.append(repr(_))
#
#     if condition["status"]:
#         _ = str(condition["status"]).strip()
#         if condition["IsFuzzy"]:
#             condition_list.append("INSTR(status, {})")
#             condition_param.append(repr(_))
#         else:
#             condition_list.append("status = {}")
#             condition_param.append(repr(_))
#
#     if condition["id"]:
#         _ = str(condition["id"]).strip()
#         condition_list.clear()  # id享有最高唯一筛选权，且不支持模糊搜索
#         condition_list.append("id = {}")
#         condition_param.clear()
#         condition_param.append(repr(_))
#
#     if len(condition_list) > 0:
#         sql_str += " WHERE " + " AND ".join(condition_list)
#
#     if condition["OrderBy"]:
#         _ = str(condition["OrderBy"]).strip()
#         sql_str += f" ORDER BY repr(_)"
#         if condition["IsDesc"]:
#             sql_str += " DESC"
#
#     if condition["Limit"]:
#         _ = str(condition["Limit"]).strip()
#         sql_str += f" LIMIT repr(_)"
#
#     if condition["Offset"]:
#         _ = str(condition["Offset"]).strip()
#         sql_str += f" OFFSET repr(_)"
#
#     sql_str = sql_str.format(*condition_param) + ";"
#     return sql_str
#
#
# def list_images_by_filter(id, attribute, group, status, offset, limit):
#     """
#     通过各种条件，从数据库检索
#     """
#
#     query_base = "SELECT `id`, `name` FROM image_stock_mass_upload_tab"
#
#     ImageFilterCondition = {
#         "id": id,
#         "name": attribute,
#         "attribute_group_name": group,
#         "status": status,
#         "Limit": limit,
#         "Offset": offset,
#         "IsFuzzy": True,
#         "OrderBy": "",
#         "IsDesc": True
#     }
#
#     query_content = sql_condition_generate(query_base, ImageFilterCondition)
#     print(query_content)
#
#
# list_images_by_filter(0, "\'", "_ \" ", "", 0, 0)


# 待确认需求：
# #所有输入是否必须字符串，不是字符串如何处理，prefix/postfix如果为空该怎样处理，最终返回结果是否去重

# 需考虑情况（测试用例）：
# string空值，返回空
# prefix空值，postfix有值：要么只返回所有postfix本身（如果存在），要么返回所有从第一个字符开始到所有postfix的子串，要么返回所有以postfix结尾的子串
# postfix空值，prefix有值：要么只返回所有prefix本身（如果存在），要么返回所有从所有prefix开始到最后一个字符的子串，要么返回所有以prefix开头的子串
# prefix和postfix都是空，返回string本身，或者返回空
# prefix和postfix相同，这个值本身也要返回
# 要考虑prefix/postfix本身是重复字符的情形（所以要逐个字符比较）
# 要考虑prefix结尾和postfix开头有重复字符，包含与被包含等交叉重复情况

# 设计思路：
# string为空，返回空；string首先必须既包含prefix又包含postfix，否则可以直接返回空
# 在string中分别找到两个子串的位置索引列表（子串在string种的开始索引和结束索引）
# 当prefix的开始索引和结束索引都小于等于postfix的开始索引和结束索引时，取prefix开始索引和postfix结束索引之间的字符串即可


# def index_substring(string, substring, step=0):
#     try:
#         index = string.index(substring)
#         string = string[index + 1:]
#         next_index = index_substring(string, substring, index + step + 1)
#         index_list = [index + step]
#         index_list.extend(next_index)
#         return index_list
#     except:
#         return []
#
#
# def query_substring(string, prefix='', postfix=''):
#     if not string or not isinstance(string, str) or not isinstance(prefix, str) or not isinstance(postfix, str):
#         return []
#     if prefix not in string or postfix not in string:
#         return []
#     if not prefix:
#         prefix_index = range(len(string))  # prefix_index = [0], or raise Error
#     else:
#         prefix_index = index_substring(string, prefix)
#     if not postfix:
#         postfix_index = range(len(string))  # postfix_index = [len(string)], or raise Error
#     else:
#         postfix_index = index_substring(string, postfix)
#     # prefix_index = index_substring(string, prefix) if prefix else range(len(string))
#     # postfix_index = index_substring(string, postfix) if postfix else range(len(string))
#
#     subs = []
#     for pre in prefix_index:
#         for post in postfix_index:
#             pre_end = pre + len(prefix)
#             post_end = post + len(postfix)
#             if pre <= post and pre_end <= post_end:
#                 sub = string[pre:post_end]
#                 subs.append(sub)
#
#     subs = list(set(subs))
#     return subs
#
#
# print(index_substring("abababbbaaab", "1"))
#
# print(query_substring("abababbbabaaab", "bab", "ab"))



# 登陆认证

# import requests
# from google_auth_oauthlib.flow import Flow

url = "https://dl-admin.test.shopee.sg/api/v2/shop_campaign/cd_item/list?limit=10&offset=0"
# cookies = {"SSO_C": "s-7455nsCgG-J2TuvRPRvAli"}
# res = requests.get(url)
# #
# print(res.content)


# def get_redirect_url(url):
#     # 请求头，这里我设置了浏览器代理
#     headers = {
#         'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36'}
#     # 请求网页
#     response = requests.get(url, headers=headers)
#     print(response.status_code)  # 打印响应的状态码
#     print(response.url)  # 打印重定向后的网址
#     # 返回重定向后的网址
#     return response.url
#
# redirect_uri = get_redirect_url(url)
#
# redirect_uri = get_redirect_url(redirect_uri)
#
# flow = Flow.from_client_secrets_file(
#     'client_secret_454252940472-g1qrhi6bmamkaqrn0tjbvelg8i5jht2b.apps.googleusercontent.com.json',
#     scopes=['profile', 'email'],
#     redirect_uri="https://soup.test.shopee.io/login/oauth2callback")
#
# authorization_url, state = flow.authorization_url(
#     access_type='offline',
#     include_granted_scopes='true')
# print(authorization_url)
# print(flow.redirect_uri)

# flow.fetch_token(authorization_response=authorization_url)

# res = requests.post(authorization_url, data=)
#
#
# app = flask.Flask(__name__)
#
# @app.route('/oauth2callback')
# def oauth2callback():
#     state = flask.session['state']
#     flow = Flow.from_client_secrets_file(
#         'client_secret_454252940472-g1qrhi6bmamkaqrn0tjbvelg8i5jht2b.apps.googleusercontent.com.json',
#         scopes=['https://dl-admin.test.shopee.sg'],
#         redirect_uri=flask.url_for('oauth2callback', _external=True),
#         state=state)
#
#     authorization_response = flask.request.url
#     flow.fetch_token(authorization_response=authorization_response)
#
#     credentials = flow.credentials
#     print(credentials)
#     return flask.redirect(flask.url_for('test_api_request'))
#
#
#
# from google_auth_oauthlib.flow import InstalledAppFlow
#
# flow = InstalledAppFlow.from_client_secrets_file(
#     'client_secret_454252940472-g1qrhi6bmamkaqrn0tjbvelg8i5jht2b.apps.googleusercontent.com.json',
#     scopes=['profile', 'email'])
#
# flow.run_local_server()

from os import path

# from src.pyland import config
#
# print(config.BASE_PATH)
#

# def ss():
#     from src.pyland.config import Config
#
#     print(Config().BASE_PATH)

# import sys
# sys.path.append("..")
# from config import Config
#
# print(Config().BASE_PATH)

# ss()






"""
在这里添加各种自定义的断言，断言失败抛出AssertionError就OK。
"""


def assertHTTPCode(response, code_list=None):
    res_code = response.status_code
    if not code_list:
        code_list = [200]
    if res_code not in code_list:
        raise AssertionError('响应code不在列表中！')  # 抛出AssertionError，unittest会自动判别为用例Failure，不是Error



# coding=utf8
import pytest
from ..config import Config
from ..pre_request import PRequest
from ..extractor import extract


def get_test_data(test_data_path):
    case = []
    http = []
    expected = []
    tests = Config(test_data_path).get('tests')
    for test in tests:
        case.append(test.get('case', ''))
        http.append(test.get('http', {}))
        expected.append(test.get('expected', {}))
    parameters = zip(case, http, expected)
    return case, parameters


cases, list_params = get_test_data("data/http_case_data/test_imgen.yml")


class TestImageGen():
    """
    Test Auto Banner Generation APIs.
    """
    @pytest.mark.skip()
    @pytest.mark.parametrize("case,http,expected", list(list_params), ids=cases)
    def test_requests(self, case, http, expected):
        response = PRequest().send_request(api_url=http['path'],
                                           method=http['method'],
                                           params=http['params'])
        assert extract("success", response) == expected['response']["success"]



"""
一些支持方法，比如加密等
❗️ 注意：基础脚本【config， support】，不能import更高级的脚本如logger， Sql
"""
import hashlib
from socket import socket, AF_INET, SOCK_STREAM
from zipfile import ZipFile
from os import path, listdir
import importlib
import types
import json


class EncryptError(Exception):
    pass


def sign(sign_dict, private_key=None, encrypt_way='MD5'):
    """传入待签名的字典，返回签名后字符串
    1.字典排序
    2.拼接，用&连接，最后拼接上私钥
    3.MD5加密"""
    dict_keys = sorted(sign_dict.keys())

    string = ''
    for key in dict_keys:
        if sign_dict[key] is None:
            pass
        else:
            string += '{0}={1}&'.format(key, sign_dict[key])
    string = string[0:len(string) - 1].replace(' ', '')

    return encrypt(string, salt=private_key, encrypt_way=encrypt_way)


def encrypt(string, salt='', encrypt_way='MD5'):
    u"""根据输入的string与加密盐，按照encrypt方式进行加密，并返回加密后的字符串"""
    string += salt
    if encrypt_way.upper() == 'MD5':
        hash_string = hashlib.md5()
    elif encrypt_way.upper() == 'SHA1':
        hash_string = hashlib.sha1()
    else:
        raise EncryptError('请输入正确的加密方式，目前仅支持 MD5 或 SHA1')

    hash_string.update(string.encode())
    return hash_string.hexdigest()


def unzip(abs_file, ext_path):
    zip_file = ZipFile(abs_file)
    for name in zip_file.namelist():
        zip_file.extract(name, ext_path)
    zip_file.close()


def get_suffix_list(file_path, suffix=".pkl"):
    """
    递归遍历指定目录，获取指定后缀的文件
    :param file_path: 目录或文件（如果是文件，则返回[file_path]）
    :param suffix: 后缀名
    :return: suffix_list = [ file1.suffix, file2.suffix, ]
    """
    suffix_list = []

    if path.isdir(file_path):
        for fi in listdir(file_path):
            fi_abspath = path.join(file_path, fi)
            if path.isfile(fi_abspath) and fi_abspath.endswith(suffix):
                suffix_list.append(fi_abspath)
            elif path.isdir(fi_abspath):
                suffix_list.extend(get_suffix_list(fi_abspath))

    elif path.isfile(file_path) and file_path.endswith(suffix):
        suffix_list = [file_path]

    return suffix_list


def socket_open(ip, port):
    # 检查IP和端口是否可连接
    s = socket(AF_INET, SOCK_STREAM)
    try:
        s.connect((ip, int(port)))
        s.shutdown(2)

        print("服务器连接成功")
        return True
    except:
        print("服务器连接失败")
        return False


def get_access_dir(dir_info, default_dir):
    # 从命令行读取测试数据路径，并检查路径是否存在
    while True:
        access_data_dir = str(input("请输入{}路径(默认当前目录下{}，输入0跳过)：".format(dir_info, default_dir)))
        if access_data_dir == "0":
            break

        if not access_data_dir:
            access_data_dir = default_dir
        else:
            access_data_dir = path.abspath(access_data_dir)

        if not path.exists(access_data_dir):
            print("路径不存在，重新输入")
        else:
            break

    return access_data_dir


def to_list(data):
    # ensure the value list is this format: [value]
    if not data:
        value_list = [None]
    elif isinstance(data, list):
        value_list = data
    else:
        value_list = [data]
    return value_list


def eval_param(data: str):
    """
    data = "${get_hash('79x79_999k')}"  -->  return get_hash('79x79_999k')
    default module is test_common.argv_read, can auto import function get_hash from file test_common/common_argv.py
    if data = "${utils.support.get_hash()}", will import get_hash from file utils/support.py
    """

    if isinstance(data, str) and data.startswith("${") and data.endswith("}"):
        data = data.lstrip("${").rstrip("}")

        func_dir = data.split("(")[0]
        func_name = func_dir.split(".")[-1]
        param = data.strip(func_dir).strip("(").strip(")")
        param_list = [i.strip() for i in param.split(",")]
        # print(f"analysis param:\t{param}")

        if func_dir == func_name:
            module = "common.common_argv"
        else:
            module = func_dir.replace(f".{func_name}", "")

        module = importlib.import_module(module)
        module = importlib.reload(module)

        for name, item in vars(module).items():
            if isinstance(item, types.FunctionType) and name == func_name:
                data = item(*param_list) if param else item()
                break
    return data


def rec_merge(d1, d2):
    """
    递归合并字典（最小单元key的value以d2为准）
    :param d1: {"a": {"c": 2, "d": 1}, "b": 2}
    :param d2: {"a": {"c": 1, "f": {"zzz": 2}}, "c": 3, }
    :return: {'a': {'c': 1, 'd': 1, 'f': {'zzz': 2}}, 'b': 2, 'c': 3}
    """
    if isinstance(d1, dict) and isinstance(d2, dict):
        if d1 == d2:
            return d1
        else:
            for key, value in d2.items():
                if key not in d1:
                    d1[key] = value
                else:
                    if isinstance(d1[key], dict) and isinstance(value, dict):
                        rec_merge(d1[key], value)
                    else:
                        d1[key] = value
            return d1
    else:
        return None


if __name__ == '__main__':
    print(encrypt('Crowd@ad123', '', 'MD5'))
    zjk = {'ZJK': 'YES', 'OK': '365  DAYS', 'WHO': 'EVERYONE', 'WHERE': 'SEA', 'DAYS': 'SOME', 'PASS': None}
    print(sign(zjk, private_key='张'))
    print(eval_param("${tests.SPDL.cms.image_svc.get_hash(79x79_999k)}"))
    print(get_suffix_list("/", ".txt"))
    de = {'DEMO/test_suits/test_01_demo.py::TestDemo::test_demo_invalid': {'case_status': 'failed', 'steps': [{'name': 'List Demo InValid', 'case_status': 'broken'}, {'name': 'List Demo InValid', 'case_status': 'broken'}, {'name': 'List Demo InValid', 'case_status': 'broken'}, {'name': 'List Demo InValid', 'case_status': 'broken'}, {'name': 'List Demo InValid', 'case_status': 'broken'}, {'name': 'List Demo InValid', 'case_status': 'broken'}, {'name': 'List Demo InValid', 'case_status': 'broken'}, {'name': 'List Demo InValid', 'case_status': 'broken'}]}, 'DEMO/test_suits/test_01_demo.py::TestDemo::test_demo_valid': {'case_status': 'failed', 'steps': [{'name': 'List Demo Valid', 'case_status': 'broken'}, {'name': 'List Demo Valid', 'case_status': 'broken'}, {'name': 'List Demo Valid', 'case_status': 'broken'}, {'name': 'List Demo Valid', 'case_status': 'broken'}]}}

    sr = {'DEMO/test_suits/test_01_demo.py::TestDemo::test_demo_invalid': {'case_id': 1}}
    rec_merge(de, sr)
    print(de)



# coding=utf8
"""
generate cases mapping relationship
upstream param:
{
    "project_id": "",
    "project_path": ""
}
call case/many api param:
[
  {
    "primary_key": "anyitem or extra.xxxx",
    "author": "string",
    "name": "string",
    "category": "string",
    "priority": "string",
    "description": "string",
    "project_id": 0,
    "group_id": 0,
    "extra": {}
  }
]
"""

import os
import sys
from subprocess import Popen, PIPE
from .config import Config, ROOT_PATH, TEST_PATH
from .log import logger
from .extractor import extract
from .thirdparty.case_manage import CaseManage


PROJECT_JSON = {
    "project_id": 35,
    "project_path": "DEMO"
}


# # 读取输入参数 PR_LIST_JSON
# PR_LIST_JSON = PR_LIST_JSON
# PR_PATH = extract("project_path", PR_LIST_JSON)
# # 系统路径配置（基础路径、测试执行路径、项目空间）
# TEST_PATH = os.path.join(BASE_PATH, "tests")
# PR_PATH = os.path.join(TEST_PATH, PR_PATH)
# sys.path.extend([BASE_PATH, TEST_PATH, PR_PATH])
# # 额外路径配置（数据路径、allure结果路径、allure报告路径）
# DATA_PATH = os.path.join(PR_PATH, "data")
# # 进入测试执行路径
# os.chdir(TEST_PATH)


class UpdateCase(object):
    """批量更新用例到平台"""

    def get_base_paths(self, project_json=PROJECT_JSON):
        """
        0 获取基础路径，准备执行环境
        """
        project_path = project_json["project_path"]
        self.project_id = project_json["project_id"]
        self.project_path = os.path.join(TEST_PATH, project_path)
        self.data_path = os.path.join(self.project_path, "data")
        sys.path.extend([ROOT_PATH, TEST_PATH, self.project_path])

    def get_case_base_info(self, project_path=None):
        """
        1. 获取用例映射关系等基本信息， 以nodeID为判断基准
        """
        project_path = project_path if project_path else self.project_path
        case_base_info = []

        os.chdir(TEST_PATH)
        with Popen(['pytest',
                    '--collect-only',
                    '--case-manage',
                    '--tb=short',  # shorter traceback format
                    project_path
                    ], stdout=PIPE, bufsize=1,
                   universal_newlines=True) as p:
            for line in p.stdout:
                print(line, end='')

        case_manage_yml = os.path.join(project_path, "data/.tmp_case_manage.yml")
        if os.path.exists(case_manage_yml):
            case_base_info = Config(case_manage_yml).get()
            os.remove(case_manage_yml)
        logger.info(f"case_base_info: {case_base_info}")
        return case_base_info

    def get_case_info(self, case_base_info):
        """
        2. 补全case信息（根据case的三级模块，判断并创建三级分组;暂时不用）
        """
        # for case in case_base_info:
        #     epic = case["extra"].get("epic")
        #     feature = case["extra"].get("feature")
        #     story = case["extra"].get("story")
        #
        #     mum_id = 1
        #     for group in [epic, feature, story]:
        #         project_name =

        for case in case_base_info:
            case.update({"project_id": self.project_id})
        case_info = case_base_info
        logger.info(f"本次发现{len(case_info)}条测试用例")
        return case_info

    def update_case_to_platform(self, case_info):
        """
        3 更新用例到平台
        """
        cm = CaseManage()
        res = cm.import_case(cases=case_info)
        return res

    def get_case_node_list(self, project_path):
        """
        4 获取用例的nodeid列表, 仅供用例执行的查询
        """
        project_path = project_path if project_path else self.project_path

        case_base_info = self.get_case_base_info(project_path)
        case_node_list = []
        for case in case_base_info:
            nodeID = extract("extra.nodeID", case).strip().replace(": ", ":")
            case_node_list.append(nodeID)
        return case_node_list


def update_case(project_json=PROJECT_JSON):
    uc = UpdateCase()
    uc.get_base_paths(project_json)
    case_base_info = uc.get_case_base_info()
    case_info = uc.get_case_info(case_base_info)
    res = uc.update_case_to_platform(case_info)
    return res


def list_case(project_json=PROJECT_JSON):
    logger.info(project_json)
    uc = UpdateCase()
    uc.get_base_paths(project_json)
    case_base_info = uc.get_case_base_info()
    case_info = uc.get_case_info(case_base_info)
    return case_info


if __name__ == "__main__":
    """传入PR_LIST_JSON = {project_path、project_id、group_id}就可以了"""
    # update_case(PROJECT_JSON)
    list_case(PROJECT_JSON)



# coding=utf8
from .config import Config
from .utils.client import HTTPClient
from .log import logger
from .extractor import extract


class PRequest(object):
    def __init__(self, base_url=None, conf='config.yml', host='host_url'):
        self.conf = conf
        self.host = host
        self.base_url = Config(self.conf).get(self.host, index=0) if not base_url else base_url
        self.headers = {}
        # self.headers = {"cookie": Config("data/cookies").raw_get()}

    def send_request(self, api_url, method='POST', status='PASS', params=None, data=None, json=None, files=None,
                     extractor=None):
        """请求定制封装"""

        # 建立http连接，发送请求
        client = HTTPClient(url=(self.base_url + api_url), method=method, headers=self.headers)
        response = client.send(params=params, json=json, data=data, files=files)
        try:
            # 有效用例和无效用例的最基本断言（针对status_code）
            if status == 'PASS':
                assert response.status_code in [200]
            elif status == 'FAIL':
                assert response.status_code not in [200]

            # 提取json值
            if status == 'PASS' and extractor is not None:
                response = extract(extractor, response.text)
                logger.info("数据处理: 提取{}的值为{} ".format(extractor, response))
        except Exception as e:
            logger.error(e)
        client.close()
        return response




# coding=utf8
"""
接口封装成类，供测试用例调用
同时，提供配置和参数的文件路径为全局参数
"""
from ..pre_request import PRequest
from ..config import Config

# # 基础路径配置， 请勿更改
# BASE_PATH = path.dirname(path.abspath(__file__))
# DATA_PATH = path.join(BASE_PATH, 'data')
# CONFIG_PATH = path.join(BASE_PATH, 'config')
# CONFIG_FILE = path.join(CONFIG_PATH, "config.yml")
# CONFIGS = Config(CONFIG_FILE).get()
# # 若不需配置cookie，可以注释这两行
# COOKIE_FILE = path.join(DATA_PATH, "input_loads/cookies")
# COOKIES = Config(COOKIE_FILE).raw_get()
#
# # 参数路径配置，按需修改
# PARAM_DEMO_FILE = path.join(DATA_PATH, "input_loads/input_demo.yml")
# # URL和SQL信息配置，按需修改
# BASE_URL = CONFIGS["dl_admin_url_id_test"]
# SQL_SERVER = CONFIGS["discover_services_id_test"]


CASE_API_INFO = {
    'list_case': ['api/cases', 'GET'],
    'create_case': ['api/cases', 'POST'],
    'detail_case': ['api/cases/{id}', 'GET'],
    'update_case': ['api/cases/{id}', 'PUT'],
    'delete_case': ['api/cases/{id}', 'DELETE'],
    'import_case': ['api/cases/many', 'POST'],
}


class CaseManage(PRequest):
    """
    Demo APIs.
    """

    def __init__(self):
        BASE_URL = Config().get("platform_url")
        super(CaseManage, self).__init__(base_url=BASE_URL)

    def import_case(self, cases):
        res = self.send_request(*CASE_API_INFO["import_case"],  json=cases)
        return res.json()

    def list_case(self, cases):
        res = self.send_request(*CASE_API_INFO["list_case"], params=cases)
        return res.json()

    def create_case(self, cases):
        res = self.send_request(*CASE_API_INFO["create_case"], params=cases)
        return res.json()

    def detail_case(self, cases):
        res = self.send_request(*CASE_API_INFO["detail_case"], params=cases)
        return res.json()

    def update_case(self, cases):
        res = self.send_request(*CASE_API_INFO["update_case"], params=cases)
        return res.json()

    def delete_case(self, cases):
        res = self.send_request(*CASE_API_INFO["delete_case"], params=cases)
        return res.json()



# src/api/routes/authors.py
from flask import Blueprint
from flask import request
import sys

sys.path.append('..')
from utils.responses import response_with
from utils import responses as resp
from cases_execute import run_plan

test_routes = Blueprint("test_routes", __name__)


@test_routes.route('/run', methods=['POST'])
def test_cases():
    try:
        data = request.get_json()
        result = run_plan(data)
        return response_with(resp.SUCCESS_201, value={"result":
                                                          result})
    except Exception as e:
        print(e)
        return response_with(resp.INVALID_INPUT_422)



# src/api/utils/responses.py
from flask import make_response, jsonify

# src/api/utils/responses.py
INVALID_FIELD_NAME_SENT_422 = {
    "http_code": 422,
    "code": "invalidField",
    "message": "Invalid fields found"
}

INVALID_INPUT_422 = {
    "http_code": 422,
    "code": "missingParameter",
    "message": "Missing parameters"
}

BAD_REQUEST_400 = {
    "http_code": 400,
    "code": "badRequest",
    "message": "Bad request"
}

SERVER_ERROR_500 = {
    "http_code": 500,
    "code": "serverError",
    "message": "Server error"
}

SERVER_ERROR_404 = {
    "http_code": 404,
    "code": "notFound",
    "message": "Resource not found"
}

UNAUTHORIZED_403 = {
    "http_code": 403,
    "code": "notAuthorized",
    "message": "You are not authorized"
}

SUCCESS_200 = {
    'http_code': 200,
    'code': 'success',
}

SUCCESS_201 = {
    'http_code': 201,
    'code': 'success',
}
SUCCESS_204 = {
    'http_code': 204,
    'code': 'success'
}


def response_with(response, value=None, message=None,
                  error=None, headers={}, pagination=None):
    result = {}
    if value is not None:
        result.update(value)

    if response.get('message', None) is not None:
        result.update({'message': response['message']})

    result.update({'code': response['code']})

    if error is not None:
        result.update({'errors': error})

    if pagination is not None:
        result.update({'pagination': pagination})

    headers.update({'Access-Control-Allow-Origin': '*'})
    headers.update({'server': 'Flask REST API'})

    return make_response(jsonify(result), response['http_code'], headers)



# coding=utf8
"""
接口封装成类，供测试用例调用
同时，提供配置和参数的文件路径为全局参数
"""
from ..pre_request import PRequest
from ..config import Config

# # 基础路径配置， 请勿更改
# BASE_PATH = path.dirname(path.abspath(__file__))
# DATA_PATH = path.join(BASE_PATH, 'data')
# CONFIG_PATH = path.join(BASE_PATH, 'config')
# CONFIG_FILE = path.join(CONFIG_PATH, "config.yml")
# CONFIGS = Config(CONFIG_FILE).get()
# # 若不需配置cookie，可以注释这两行
# COOKIE_FILE = path.join(DATA_PATH, "input_loads/cookies")
# COOKIES = Config(COOKIE_FILE).raw_get()
#
# # 参数路径配置，按需修改
# PARAM_DEMO_FILE = path.join(DATA_PATH, "input_loads/input_demo.yml")
# # URL和SQL信息配置，按需修改
# BASE_URL = CONFIGS["dl_admin_url_id_test"]
# SQL_SERVER = CONFIGS["discover_services_id_test"]


GROUP_API_INFO = {
    'list_group': ['api/groups', 'GET'],
    'create_group': ['api/groups', 'POST'],
    'detail_group': ['api/groups/{id}', 'GET'],
    'update_group': ['api/groups/{id}', 'PUT'],
    'delete_group': ['api/groups/{id}', 'DELETE']
}


class GroupManage(PRequest):
    """
    Demo APIs.
    """

    def __init__(self):
        BASE_URL = Config().get("platform_url")
        super(GroupManage, self).__init__(base_url=BASE_URL)

    def list_group(self, groups):
        res = self.send_request(*GROUP_API_INFO["list_group"],  params=groups)
        return res

    def create_group(self, groups):
        res = self.send_request(*GROUP_API_INFO["create_group"],  params=groups)
        return res

    def detail_group(self, groups):
        res = self.send_request(*GROUP_API_INFO["detail_group"],  params=groups)
        return res

    def update_group(self, groups):
        res = self.send_request(*GROUP_API_INFO["update_group"],  params=groups)
        return res

    def delete_group(self, groups):
        res = self.send_request(*GROUP_API_INFO["delete_group"],  params=groups)
        return res





# coding=utf8
import shutil
from os import path


def copy_history(report, result):
    start_path = path.join(report, 'history')
    end_path = path.join(result, 'history')
    if path.exists(end_path):
        shutil.rmtree(end_path)
    try:
        shutil.copytree(start_path, end_path)
        print("复制上一运行结果成功！")
    except FileNotFoundError:
        print("allure没有历史数据可复制！")



# src/api/routes/authors.py
from flask import Blueprint
from flask import request
from ..utils.responses import response_with
from ..utils import responses as resp
from ..cases_collect import list_case, update_case

case_routes = Blueprint("case_routes", __name__)


@case_routes.route('/update', methods=['POST'])
def update_cases():
    try:
        data = request.get_json()
        result = update_case(data)
        return response_with(resp.SUCCESS_201, value={"result":
                                                          result})
    except Exception as e:
        print(e)
        return response_with(resp.INVALID_INPUT_422)


@case_routes.route('/list', methods=['GET'])
def list_cases():
    try:
        data = request.get_json()
        result = list_case(data)
        return response_with(resp.SUCCESS_201, value={"result":
                                                          result})
    except Exception as e:
        print(e)
        return response_with(resp.INVALID_INPUT_422)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyland",
    version="0.0.1",
    author="jiukun",
    author_email="jiukun9291@gmail.com",
    description="use yaml to combination params for pytest collect test cases",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    project_urls={
        "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # entry_points={
    #     'console_scripts': [''],
    # },
    package_dir={"": "src"},
    package_data={
            "": ["*.yml"],
    },
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
    install_requires=[
        "pytest",
        "allure-pytest",
        "requests",
        "PyYAML",
        "xlwt",
        "xlrd",
        "mysql-connector-python",
        "jmespath",
        "selenium",
        "Faker",
        "Pillow",
        "pypng",
        "numpy",
        "sqlmap",
        "pytest-xdist",
        "Flask",
        "Jinja2",
        "flask_migrate",
        "flask_cors",
        "Flask-SQLAlchemy",
        "flask_migrate",
        "flask_restful",
        "fnv",
    ]
)



"""
读取配置。采用yaml格式配置文件，也可以采用xml、ini等，需要在file_rader添加响应Reader处理。
❗️ 注意：基础脚本【config， support】，不能import更高级的脚本如logger， Sql
"""
import sys, os
from .utils.file_reader import YamlReader, JsonReader, RawReader
from .utils.file_writer import YamlWriter, JsonWriter, RawWriter
from .utils.support import to_list, eval_param

# BASE_PATH = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]

BASE_PATH = os.getcwd()
CONFIG_PATH = os.path.join(BASE_PATH, 'config')
DATA_PATH = os.path.join(BASE_PATH, 'data')
DRIVERS_PATH = os.path.join(BASE_PATH, 'drivers')
LOG_PATH = os.path.join(BASE_PATH, 'log')
REPORT_PATH = os.path.join(BASE_PATH, 'reports')
CONFIG_FILE = os.path.join(CONFIG_PATH, 'config.yml')

ROOT_PATH = os.path.split(os.path.dirname(BASE_PATH))[0]  # ./
TEST_PATH = os.path.join(ROOT_PATH, "tests")


class Config:
    """
    读取yaml配置文件的元素
    Config(): 默认读取 ${workspace}/config/config.yml
    Config('api_config.yml'): 读取 ${workspace}/config/api_config.yml
    Config('testAPI/interface/mod_2_video/test.yml'): 读取 ${workspace}/testAPI/interface/mod_2_video/test.yml
    Config('/usr/data/images'): 读取 /usr/data/images
    """

    def __init__(self, config='config.yml', index=0):
        if config.startswith("/"):
            self.abspath = config
        elif '/' in config:
            self.abspath = BASE_PATH
            for var in config.split('/'):
                self.abspath = os.path.join(self.abspath, var)
        else:
            self.abspath = os.path.join(CONFIG_PATH, config)

        try:
            self.config = self.get(index=index)
        except Exception as e:
            self.config = None

    def get(self, element=None, index=0):
        """
        get config element
        :param element: yaml element
        :param index: yaml is separated to lists by '---'
        :return:
        """
        try:
            self.config = YamlReader(self.abspath).data[index]
        except Exception as e:
            raise Exception("Yaml read error")

        if element:
            res = self.config.get(element)
        else:
            res = self.config
        return res

    def global_var(self, index=0):
        """
        Be careful! Transfer yaml key to global var name
        :return:
        """
        if index:
            self.config = YamlReader(self.abspath).data[index]

        for key in sorted(self.config.keys()):
            globals()[key] = self.config[key]



    def json_get(self):
        """从json文件读取json对象"""
        json_data = JsonReader(self.abspath).data
        return json_data

    def json_put(self, json_obj, abspath=None):
        """把json对象写为json文件"""
        abspath = self.abspath if not abspath else abspath
        self.touch(abspath)
        JsonWriter(json_obj, self.abspath)

    def json_update(self):
        pass

    def raw_get(self):
        raw_data = RawReader(self.abspath).data
        return raw_data

    def raw_put(self, raw_obj):
        RawWriter(raw_obj, self.abspath)

    @classmethod
    def touch(cls, abspath):
        """当写入文件时，需要保证路径和文件都是存在的"""
        dirname = os.path.dirname(abspath)
        os.makedirs(dirname, exist_ok=True)
        with open(abspath, 'a'):
            os.utime(abspath, None)
            
    @classmethod
    def put(cls, yaml_obj, abspath):
        """把yaml格式对象，写入一个新的yaml文件（覆盖重写）"""
        Config.touch(abspath)
        YamlWriter(yaml_obj, abspath, overwrite=True)

    @classmethod
    def update(cls, yaml_obj, abspath=None):
        """把yaml格式对象，写入一个旧的yaml文件（更新追加）"""
        try:
            dest = YamlReader(abspath).data[0]
            if isinstance(dest, list) and isinstance(yaml_obj, list):
                YamlWriter(yaml_obj, abspath, overwrite=False)
            elif isinstance(dest, dict) and isinstance(yaml_obj, dict):
                dest.update(yaml_obj)
                YamlWriter(dest, abspath, overwrite=True)
            else:
                raise ValueError("试图写入不同类型的数据到YAML文件，请检查！ ")
        except FileNotFoundError:
            cls.put(yaml_obj, abspath)



class YamlParam(Config):
    """
    Read and Format Input Param from config
    """

    @staticmethod
    def parse_by_case_type(original_data, type='valid'):
        # transfer `{a: {valid: []}, b: {valid: []}}` to `{a: [], b: []}`
        original_data = eval_param(original_data)
        param = {}
        param_list = []

        if isinstance(original_data, dict):
            type_value = original_data.get(type)
            if type_value:
                type_value = to_list(type_value)
                param_list = YamlParam.parse_by_case_type(type_value, type)
                return param_list
            else:
                for key in original_data.keys():
                    if key != 'valid' and key != 'invalid':
                        param[key] = YamlParam.parse_by_case_type(original_data.get(key), type)
                return param

        elif isinstance(original_data, list):
            for value in original_data:
                param = YamlParam.parse_by_case_type(value, type)
                param_list.append(param)
            return param_list
        else:
            return original_data

    @staticmethod
    def dict_value_to_list(src_dict: dict):
        """
        To transfer a dict to a permutation lists
        :param src_dict: {key1: [a1,a2], key2: [b1,b2,b3], key3:[c1]}
        :return: [[a1,b1,c1], [a2,b2,c1], [a2,b3,c1]]
        """
        res = []
        over_key = set()
        while len(over_key) < len(src_dict):
            lr = len(res)
            res_i = {}
            for key in src_dict.keys():
                # ensure the value list is this format: [value]
                if src_dict[key] and isinstance(src_dict[key], list):
                    value_list = src_dict[key]
                else:
                    value_list = [src_dict[key]]

                # combine valid params
                if lr < len(value_list) - 1:
                    res_i[key] = value_list[lr]
                else:
                    over_key.add(key)
                    res_i[key] = value_list[-1]

                # disable key if value is 'x'
                if res_i[key] == 'x':
                    res_i.pop(key)
            if res_i not in res:
                res.append(res_i)
        return res

    @staticmethod
    def replace_json_with_dict(src_dict: dict, json: dict) -> list:
        """
        To combine yml wrong params with the template
        :param src: {arg1: [1,2,3]}
        :param json: {arg1: 0, arg2:9}
        :return: [{arg1: 1, arg2:9},{arg1: 2, arg2:9},{arg1: 3, arg2:9}]
        """
        res = []
        for key in src_dict.keys():
            # ensure the value list is this format: [value]
            if src_dict[key] and isinstance(src_dict[key], list):
                value_list = src_dict[key]
            else:
                value_list = [src_dict[key]]

            for val in value_list:
                tmp = json.copy()
                tmp[key] = val

                # disable key if value is 'x'
                if val == 'x':
                    tmp.pop(key)

                if tmp not in res:
                    res.append(tmp)
        return res

    def valid_params(self, element):
        """
        获取组合后的·有效字段·数据列表
        """
        # ele_valid = self.get(element).get('valid')
        ele = self.get(element)
        ele_valid = self.parse_by_case_type(ele, type='valid')
        if ele_valid and isinstance(ele_valid, dict):
            ele_valid = self.dict_value_to_list(ele_valid)
        self.ele_valid = ele_valid
        return self.ele_valid

    def invalid_params(self, element, template_file=None):
        """
        获取组合后的·无效字段·数据列表
        """
        # ele_invalid = self.get(element).get('invalid')
        ele = self.get(element)
        ele_invalid = self.parse_by_case_type(ele, type='invalid')
        if ele_invalid and isinstance(ele_invalid, dict):
            try:
                if template_file:
                    template = Config(template_file).json_get()
                elif self.valid_params(element) and isinstance(self.ele_valid[0], dict):
                    template = self.ele_valid[0]
                ele_invalid = self.replace_json_with_dict(ele_invalid, template)
            except Exception as e:
                raise FileNotFoundError("need template json file")
        self.ele_invalid = ele_invalid
        return self.ele_invalid

    def combined_params(self):
        """
        组合该yml文件下所有param， 生成_combined_xx.yml文件
        并返回其字典格式数据
        """
        res = {}
        try:
            all_params = self.config
            for key in all_params.keys():
                if key.startswith("param"):
                    valid = self.valid_params(key)
                    invalid = self.invalid_params(key)
                    res.update({f"{key}_valid": valid, f"{key}_invalid": invalid})
            original_path = os.path.split(self.abspath)
            out_path = os.path.join(original_path[0], "_combined_" + original_path[1])
            self.put(res, out_path)
        except Exception as e:
            raise
        return res


def com_params(yaml_files_list):
    """
    根据yaml文件列表，返回一个输入参数字典
    """
    param_dict = {}
    for file in yaml_files_list:
        yp = YamlParam(file)
        combined = yp.combined_params()
        param_dict.update(combined)
    return param_dict


# CONFIGS = Config(CONFIG_FILE).get()



"""用JMESPath实现的抽取器，对于json格式数据实现简单方式的抽取"""
from jmespath import search as jmes_search
from requests.models import Response
from json import loads as j_loads
from json import dumps as j_dumps


def extract(query=None, body=None):
    try:
        if type(body) is Response:
            body = body.json()
        if type(body) is str:
            body = j_loads(body)
        return jmes_search(query, body)
    except Exception as e:
        raise ValueError("Invalid query for `" + query + "`: " + str(e))


if __name__ == '__main__':
    from src.pyland.api import HTTPClient

    res = HTTPClient(url='http://wthrcdn.etouch.cn/weather_mini?citykey=101010100', method='GET').send()
    print(j_dumps(res.json(), indent=2, ensure_ascii=False))
    j_1 = extract(query='data.forecast[0].date', body=res.json())
    j_2 = extract(query='data.ganmao', body=res)
    print(j_1 + '\n' + j_2)
